
#include <catch2/catch_test_macros.hpp>
#include "aidge/graphRegex/GraphRegex.hpp"

#include "aidge/operator/Conv.hpp"
#include "aidge/operator/GenericOperator.hpp"

using namespace Aidge;

TEST_CASE("GraphRegexUser") {

    SECTION("INIT") {

        const std::string query = "Conv->FC";

        std::shared_ptr<GraphRegex> sut = std::make_shared<GraphRegex>();

        std::shared_ptr<GraphView> g1 = std::make_shared<GraphView>("TestGraph");
        std::shared_ptr<Node> conv = GenericOperator("Conv", 1, 1, 1, "c");
        std::shared_ptr<Node> fc = GenericOperator("FC", 1, 1, 1, "c1");
        std::shared_ptr<Node> conv2 = GenericOperator("Conv", 1, 1, 1, "c2");
        std::shared_ptr<Node> fc2 = GenericOperator("FC", 1, 1, 1, "c3");

        g1->add(conv);
        g1->addChild(fc, "c");
        g1->addChild(conv2, "c1");
        g1->addChild(fc2, "c2");


        sut->setKeyFromGraph(g1);
        sut->addQuery(query);
        
        for (const auto& solution : sut->match(g1)) {

            REQUIRE(solution->getQuery() == query);
            if(solution->getStartNode() == std::vector<NodePtr>{conv}){
                REQUIRE(solution->at("Conv") == std::set<NodePtr>{conv} );
                REQUIRE(solution->at("FC") == std::set<NodePtr>{fc} );
            }else if (solution->getStartNode() == std::vector<NodePtr>{conv2})
            {
                REQUIRE(solution->at("Conv") == std::set<NodePtr>{conv2} );
                REQUIRE(solution->at("FC") == std::set<NodePtr>{fc2} );
            }
        }
        //REQUIRE( sut->match(g1)[1]->getAll() == std::set<NodePtr>{conv,fc});
        
    }

   SECTION("CC") {

        

        std::shared_ptr<GraphRegex> sut = std::make_shared<GraphRegex>();


        std::shared_ptr<GraphView> g1 = std::make_shared<GraphView>("TestGraph");
        std::shared_ptr<Node> conv = GenericOperator("Conv", 1, 1, 1, "c");
        std::shared_ptr<Node> conv1 = GenericOperator("Conv", 1, 1, 1, "c1");
        std::shared_ptr<Node> conv2 = GenericOperator("Conv", 1, 1, 1, "c2");
        std::shared_ptr<Node> conv3 = GenericOperator("Conv", 1, 1, 1, "c3");

        g1->add(conv);
        g1->addChild(conv1, "c");
        g1->addChild(conv2, "c1");
        g1->addChild(conv3, "c2");


        sut->setKeyFromGraph(g1);

        const std::string query = "Conv->Conv";
        const std::string query2 = "Conv->FC";

        sut->setNodeKey("FC","getType($) =='FC'");

        sut->addQuery(query);
        sut->addQuery(query2);


        for (const auto& solution : sut->match(g1)) {
            REQUIRE(solution->getQuery() == query);
        }
        
    }
}